searchNodes=[{"doc":"Generates definitions of various things related to tuples with a tagged value, such as the ubiquitous {:ok, value} and {:error, reason}.Examplesdefmodule Tagged.Status use Tagged deftagged ok deftagged error endConstruct and Destructureiex&gt; require Tagged.Status, as: Status iex&gt; Status.ok(:computer) {:ok, :computer} iex&gt; with Status.error(reason) &lt;- {:ok, :computer}, do: raise reason {:ok, :computer}See Tagged.Constructor for further details.Type definitions_iex&gt; t Tagged.Status.error @type error() :: {:error, term()} Tagged value tuple, containing term().See Tagged.Typedef for further details.Pipe selective executioniex&gt; require Tagged.Status iex&gt; import Tagged.Status, only: [ok: 1, with_ok: 2] iex&gt; ok(:computer) |&gt; with_ok(&amp; &quot;OK, \#{&amp;1}&quot;) &quot;OK, computer&quot;See Tagged.PipeWith for further details.","ref":"Tagged.html","title":"Tagged","type":"module"},{"doc":"Generates a macro that definies all things related to a tagged value tuple, {atom(), term()}. By default the macro has the same name as the tag, and all the things are generated.Keywordsas: nameOverride default macro name. See Tagged.Constructor.of: typedefDeclare the wrapped type statically, making it opaque. See Tagged.Typedef.type: falseOverride generation of type definition. See Tagged.Typedef.pipe_with: falseOverride generation of pipe filter. See Tagged.PipeWith.","ref":"Tagged.html#deftagged/2","title":"Tagged.deftagged/2","type":"macro"},{"doc":"Generates macros for constructing, and destructuring tagged value tuples. By default, the macro name is the same as the tag, but can be overriden with the as: name keyword argument.This module is always executed.ExamplesDefine constructors with the same name as the tagdefmodule Tagged.Status use Tagged deftagged ok deftagged error end iex&gt; require Tagged.Status iex&gt; import Tagged.Status, only: [ok: 1] iex&gt; ok(:computer) {:ok, :computer} iex&gt; with ok(it) &lt;- Keyword.fetch([a: &quot;bacon&quot;], :a), do: &quot;Chunky \#{it}!&quot; &quot;Chunky bacon!&quot;Override constructor namedefmodule Tagged.Outcome use Tagged deftagged ok, as: success deftagged error, as: failure end iex&gt; require Tagged.Outcome iex&gt; import Tagged.Outcome, only: [failure: 1, success: 1] iex&gt; failure(:is_human) {:error, :is_human} iex&gt; with success(it) &lt;- {:ok, &quot;Computer&quot;}, do: &quot;OK, \#{it}!&quot; &quot;OK, Computer!&quot;","ref":"Tagged.Constructor.html","title":"Tagged.Constructor","type":"module"},{"doc":"Generates macros for use in guard expressions.Examplesiex&gt; require Tagged.Outcome iex&gt; import Tagged.Outcome iex&gt; f = fn x when is_success(x) -&gt; x; _ -&gt; success(nil) end iex&gt; success(:computer) |&gt; f.() {:ok, :computer} iex&gt; failure(:computer) |&gt; f.() {:ok, nil}","ref":"Tagged.Guard.html","title":"Tagged.Guard","type":"module"},{"doc":"Reasoning in terms of the outcome of an action.iex&gt; require Tagged.Outcome iex&gt; import Tagged.Outcome, only: [failure: 1, success: 1] iex&gt; failure(:is_human) {:error, :is_human} iex&gt; with success(it) &lt;- {:ok, &quot;Computer&quot;}, do: &quot;OK, \#{it}!&quot; &quot;OK, Computer!&quot;","ref":"Tagged.Outcome.html","title":"Tagged.Outcome","type":"module"},{"doc":"Constructor for error tagged value tuples. Can also be used to destructure tuples.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; with failure(val) &lt;- {:error, :match}, do: val :match iex&gt; with failure(_) &lt;- {:not_error, :match}, do: true {:not_error, :match}","ref":"Tagged.Outcome.html#failure/1","title":"Tagged.Outcome.failure/1","type":"macro"},{"doc":"Guard macro for testing if term is a error tagged tuple.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; f = fn x when is_failure(x) -&gt; x; _ -&gt; nil end iex&gt; {:error, true} |&gt; f.() {:error, true} iex&gt; {:not_error, true} |&gt; f.() nil","ref":"Tagged.Outcome.html#is_failure/1","title":"Tagged.Outcome.is_failure/1","type":"macro"},{"doc":"Guard macro for testing if term is a ok tagged tuple.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; f = fn x when is_success(x) -&gt; x; _ -&gt; nil end iex&gt; {:ok, true} |&gt; f.() {:ok, true} iex&gt; {:not_ok, true} |&gt; f.() nil","ref":"Tagged.Outcome.html#is_success/1","title":"Tagged.Outcome.is_success/1","type":"macro"},{"doc":"Constructor for ok tagged value tuples. Can also be used to destructure tuples.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; with success(val) &lt;- {:ok, :match}, do: val :match iex&gt; with success(_) &lt;- {:not_ok, :match}, do: true {:not_ok, :match}","ref":"Tagged.Outcome.html#success/1","title":"Tagged.Outcome.success/1","type":"macro"},{"doc":"Calls f/1 with the wrapped value, when term matches a error tagged tuple. When term does not match, is is returned as-is.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; {:error, :match} |&gt; with_failure(&amp; &amp;1) :match iex&gt; {:not_error, :match} |&gt; with_failure(&amp; &amp;1) {:not_error, :match}","ref":"Tagged.Outcome.html#with_failure/2","title":"Tagged.Outcome.with_failure/2","type":"macro"},{"doc":"Calls f/1 with the wrapped value, when term matches a ok tagged tuple. When term does not match, is is returned as-is.iex&gt; require Elixir.Tagged.Outcome iex&gt; import Elixir.Tagged.Outcome iex&gt; {:ok, :match} |&gt; with_success(&amp; &amp;1) :match iex&gt; {:not_ok, :match} |&gt; with_success(&amp; &amp;1) {:not_ok, :match}","ref":"Tagged.Outcome.html#with_success/2","title":"Tagged.Outcome.with_success/2","type":"macro"},{"doc":"","ref":"Tagged.Outcome.html#t:failure/0","title":"Tagged.Outcome.failure/0","type":"type"},{"doc":"Tagged value tuple with a wrapped type t \\\\ term()","ref":"Tagged.Outcome.html#t:failure/1","title":"Tagged.Outcome.failure/1","type":"opaque"},{"doc":"","ref":"Tagged.Outcome.html#t:success/0","title":"Tagged.Outcome.success/0","type":"type"},{"doc":"Tagged value tuple with a wrapped type t \\\\ term()","ref":"Tagged.Outcome.html#t:success/1","title":"Tagged.Outcome.success/1","type":"opaque"},{"doc":"Generates a function for selective execution, with pass-through of terms that does not match, much like the regular with ... &lt;- ..., do: ... construct.ExamplesGiven a module that defines a success() | failure() type:defmodule DocTest.PipeWith do use Tagged deftagged not_an_integer deftagged not_a_number @type reason :: not_an_integer() | not_a_number() deftagged success deftagged failure @type result :: success(integer()) | failure(reason()) def validate_input(x) when is_integer(x), do: success(x) def validate_input(x), do: failure(not_an_integer(x)) def next_number(x), do: x + 1 def try_recovery({_, x}) when is_number(x), do: success(floor(x)) def try_recovery({_, x}), do: failure(not_a_number(x)) endThis is quite similar to the regular with ... &lt;- ..., do: ..., else: ... for happy paths:iex&gt; require DocTest.PipeWith iex&gt; import DocTest.PipeWith iex&gt; with success(v) &lt;- validate_input(1), ...&gt; do: next_number(v) 2 iex&gt; validate_input(1) ...&gt; |&gt; with_success(&amp;next_number/1) 2When the path is not a happy path, it offers more fluent control over recovery from failures at any point in the pipe:iex&gt; require DocTest.PipeWith iex&gt; import DocTest.PipeWith iex&gt; with success(v) &lt;- validate_input(0.7) do ...&gt; next_number(v) ...&gt; else ...&gt; failure(e) -&gt; with success(v) &lt;- try_recovery(e), ...&gt; do: next_number(v) ...&gt; end 1 iex&gt; validate_input(0.7) ...&gt; |&gt; with_failure(&amp;try_recovery/1) ...&gt; |&gt; with_success(&amp;next_number/1) 1","ref":"Tagged.PipeWith.html","title":"Tagged.PipeWith","type":"module"},{"doc":"Resoning in terms of the status of a result.iex&gt; require Tagged.Status iex&gt; import Tagged.Status, only: [ok: 1] iex&gt; ok(:computer) {:ok, :computer} iex&gt; with ok(it) &lt;- Keyword.fetch([a: &quot;bacon&quot;], :a), do: &quot;Chunky \#{it}!&quot; &quot;Chunky bacon!&quot;","ref":"Tagged.Status.html","title":"Tagged.Status","type":"module"},{"doc":"Constructor for error tagged value tuples. Can also be used to destructure tuples.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; with error(val) &lt;- {:error, :match}, do: val :match iex&gt; with error(_) &lt;- {:not_error, :match}, do: true {:not_error, :match}","ref":"Tagged.Status.html#error/1","title":"Tagged.Status.error/1","type":"macro"},{"doc":"Guard macro for testing if term is a error tagged tuple.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; f = fn x when is_error(x) -&gt; x; _ -&gt; nil end iex&gt; {:error, true} |&gt; f.() {:error, true} iex&gt; {:not_error, true} |&gt; f.() nil","ref":"Tagged.Status.html#is_error/1","title":"Tagged.Status.is_error/1","type":"macro"},{"doc":"Guard macro for testing if term is a ok tagged tuple.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; f = fn x when is_ok(x) -&gt; x; _ -&gt; nil end iex&gt; {:ok, true} |&gt; f.() {:ok, true} iex&gt; {:not_ok, true} |&gt; f.() nil","ref":"Tagged.Status.html#is_ok/1","title":"Tagged.Status.is_ok/1","type":"macro"},{"doc":"Constructor for ok tagged value tuples. Can also be used to destructure tuples.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; with ok(val) &lt;- {:ok, :match}, do: val :match iex&gt; with ok(_) &lt;- {:not_ok, :match}, do: true {:not_ok, :match}","ref":"Tagged.Status.html#ok/1","title":"Tagged.Status.ok/1","type":"macro"},{"doc":"Calls f/1 with the wrapped value, when term matches a error tagged tuple. When term does not match, is is returned as-is.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; {:error, :match} |&gt; with_error(&amp; &amp;1) :match iex&gt; {:not_error, :match} |&gt; with_error(&amp; &amp;1) {:not_error, :match}","ref":"Tagged.Status.html#with_error/2","title":"Tagged.Status.with_error/2","type":"macro"},{"doc":"Calls f/1 with the wrapped value, when term matches a ok tagged tuple. When term does not match, is is returned as-is.iex&gt; require Elixir.Tagged.Status iex&gt; import Elixir.Tagged.Status iex&gt; {:ok, :match} |&gt; with_ok(&amp; &amp;1) :match iex&gt; {:not_ok, :match} |&gt; with_ok(&amp; &amp;1) {:not_ok, :match}","ref":"Tagged.Status.html#with_ok/2","title":"Tagged.Status.with_ok/2","type":"macro"},{"doc":"","ref":"Tagged.Status.html#t:error/0","title":"Tagged.Status.error/0","type":"type"},{"doc":"Tagged value tuple with a wrapped type t \\\\ term()","ref":"Tagged.Status.html#t:error/1","title":"Tagged.Status.error/1","type":"opaque"},{"doc":"","ref":"Tagged.Status.html#t:ok/0","title":"Tagged.Status.ok/0","type":"type"},{"doc":"Tagged value tuple with a wrapped type t \\\\ term()","ref":"Tagged.Status.html#t:ok/1","title":"Tagged.Status.ok/1","type":"opaque"},{"doc":"Generates type definitions for tagged value tuples.This module is executed by default, but can be disabled for a whole module with keyword argument types: false to use/2, or for a single definition with keyword argument type: false to Tagged.deftagged/2.The wrapped type can be declared statically, which makes it opaque, by giving the of: keyword to Tagged.deftagged/2.ExamplesDisable type declaration for all tagged value tuple definitionsdefmodule DocTest.NoTypes do use Tagged, types: false deftagged foo end _iex&gt; use DocTest.NoTypes _iex&gt; t foo No type information for Kernel.foo was found or Kernel.foo is privateOverride type declaration for a single tagged value tuple definitiondefmodule DocTest.SomeTypes do use Tagged deftagged foo, type: false deftagged bar end _iex&gt; use DocTest.SomeTypes _iex&gt; t foo No type information for Kernel.foo was found or Kernel.foo is private _iex&gt; t bar @type bar() :: bar(term()) @opaque bar(t) Tagged value tuple with a wrapped type t() \\\\ term()Make the wrapped type static opaquedefmodule DocTest.OpaqueType do use Tagged deftagged foo, of: integer() end _iex&gt; use DocTest.OpaqueType _iex&gt; t foo @opaque foo()","ref":"Tagged.Typedef.html","title":"Tagged.Typedef","type":"module"},{"doc":"ChangelogAll notable changes to this project will be documented in this file.The format is based on Keep a Changelog, and this project adheres to Semantic Versioning.","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"AddedGenerate defguard macros.Option to declare the wrapped type statically.ChangedGenerated typedefs are opaque.","ref":"changelog.html#v0-3-0-2020-05-24","title":"Changelog - v0.3.0 - 2020-05-24","type":"extras"},{"doc":"AddedSelective function execution in pipes.ChangedOption keywords are now validated on both module and macro level.FixedModule level use options was not properly unwrapped.Generated constructor doc examples referenced the wrong module.Code formatting was not configured to keep deftagged without parenthesis.","ref":"changelog.html#v0-2-0-2020-05-22","title":"Changelog - v0.2.0 - 2020-05-22","type":"extras"},{"doc":"Initial package release.AddedDefine deftagged macro, used to declare a tagged value tuple.Generate a constructor / destructuring macro for a tag.Generate a typedef for a tag.","ref":"changelog.html#v0-1-0-2020-05-19","title":"Changelog - v0.1.0 - 2020-05-19","type":"extras"}]